#include "homogmech.h"
#include "homog.h"
#include "global.h"
#include "element.h"
#include "intpoints.h"
#include "globmat.h"
#include "gtopology.h"
#include "math.h"
#include "solverm.h"
#include "lssolver.h"
#include "mechbclc.h"
#include "loadcase.h"

/**
   function solves homogenization for mechanical problems on PUC

   TKr, 07/01/2015
*/
void mechanical_homogenization ()
{
  long i,j,ii,jj,hncomp;

  //  initiation of transport material models
  //Mm->initmaterialmodels();
  //  nodes - integration points interpolation
  //approximation_puc (); //if it is needed
  
  
  //solving of fluctuation parts
  // zatim pouze pro jeden zatezovaci stav a elasticky material:
  
  //tady doplnit cyklus pres vsechny jednotkove stavy podle typu homog. ulohy:
  Mp->stresscomp = 1; //for macrostresses computation
  Mp->straincomp = 1; //for macrostrains computation
  switch (Mp->hstrastre){
  case planestress:{
    hncomp = 3;
    break;
  }
  case planestrain:{
    hncomp = 3;
    break;
  }
  case axisymm:{
    hncomp = 4;
    break;
  }
  case spacestress:{
    hncomp = 6;
    break;
  }
  default:{
    print_err("unknown type of strain-stress state for mechanical homogeniaztion is required", __FILE__, __LINE__, __func__);
  }
  }//  end of the switch (strastre){

  //matrix allocation
  matrix d(ASTCKMAT(hncomp,hncomp));

  if(Mp->homog > 6){//options 7 or 8 for testing
    ii = Mp->homogdir-1; //unifom load direction
    
    //macro strain-stress loading:
    if (Mp->homog == 7)
      {
	for(jj=0;jj<hncomp;jj++)//null mstress
	  Mb->lc[0].mstress[jj] = 0.0;
	
	Mb->lc[0].mstress[ii] = 1.0;//unit mstress for ii components
      }
    if (Mp->homog == 8)
      {
	for(jj=0;jj<hncomp;jj++)//null mstress
	  Mb->lc[0].mstrain[jj] = 0.0;
	
	Mb->lc[0].mstrain[ii] = 1.0;//unit mstress for ii components
      }

    switch (Mp->tprob){
    case linear_statics:{
      solve_linear_statics ();
      break;
    }
    default:{
      print_err("unknown problem type is required",__FILE__,__LINE__,__func__);
    }
    }
    
    switch (Mp->homog){
    case 7:{ // macro-stress approach
      //compliance matrix computation from averaged strains
      for(i=0;i<hncomp;i++)
	d[i][ii] = macrostrains(0,i);
      break;
    }
    case 8:{ // macro-strain approach
      //stiffness matrix computation from averaged stresses (macrostress)
      for(i=0;i<hncomp;i++)
	d[i][ii] = macrostresses(0,i);
      break;
    }
    default:{
      print_err("unknown type of homogenization is required", __FILE__, __LINE__, __func__);
      abort();
    }
    }
  }
  else{
    
    //loop over strain-stress components; only for one (first) loadcase
    for(ii=0;ii<hncomp;ii++){
      
      //macro strain-stress loading:
      if (Mp->homog == 5)
	{
	  for(jj=0;jj<hncomp;jj++)//null mstress
	    Mb->lc[0].mstress[jj] = 0.0;
	  
	  Mb->lc[0].mstress[ii] = 1.0;//unit mstress for ii components
	}
      if (Mp->homog == 6)
	{
	  for(jj=0;jj<hncomp;jj++)//null mstress
	    Mb->lc[0].mstrain[jj] = 0.0;
	  
	  Mb->lc[0].mstrain[ii] = 1.0;//unit mstress for ii components
	}
      
      switch (Mp->tprob){
      case linear_statics:{
	solve_linear_statics ();
	break;
      }
      default:{
	print_err("unknown problem type is required",__FILE__,__LINE__,__func__);
      }
      }
      
      switch (Mp->homog){
      case 5:{ // macro-stress approach
	//compliance matrix computation from averaged strains
	for(i=0;i<hncomp;i++)
	  d[i][ii] = macrostrains(0,i);
	break;
      }
      case 6:{ // macro-strain approach
	//stiffness matrix computation from averaged stresses (macrostress)
	for(i=0;i<hncomp;i++)
	  d[i][ii] = macrostresses(0,i);
	break;
      }
      default:{
	print_err("unknown type of homogenization is required", __FILE__, __LINE__, __func__);
	abort();
      }
      }
    }//end of loop over strain-stress components
  }

  if (Mespr != 0){//printing of matrix into .log file
    switch (Mp->homog){
    case 5:
    case 7:{ // macro-stress approach
      fprintf(Out,"\n\n Resulting Compliance matrix C\n");
      break;
    }
    case 6:
    case 8:{ // macro-strain approach
      fprintf(Out,"\n\n Resulting Stiffness matrix D\n");
      break;
    }
    default:{
      print_err("unknown type of homogenization is required", __FILE__, __LINE__, __func__);
      abort();
    }
    }
    
    for (i=0;i<hncomp;i++){
      for (j=0;j<hncomp;j++)
	fprintf(Out,"%e  ",d[i][j]);
      fprintf(Out,"\n");
    }
    
    //printing for general anisotropic elastic material:
    switch (Mp->hstrastre){
    case planestress:
    case planestrain:{
      fprintf(Out,"\n\n Print for elastgmat2d:\n");
      fprintf(Out,"%le %le %le\n",d[0][0],d[0][1],d[0][2]);
      fprintf(Out,"%le %le\n",d[1][1],d[1][2]);
      fprintf(Out,"%le\n",d[2][2]);
      break;
    }
    case spacestress:{
      fprintf(Out,"\n\n Print for elastgmat3d:\n");
      fprintf(Out,"%le %le %le %le %le %le\n",d[0][0],d[0][1],d[0][2],d[0][3],d[0][4],d[0][5]);
      fprintf(Out,"%le %le %le %le %le\n",d[1][1],d[1][2],d[1][3],d[1][4],d[1][5]);
      fprintf(Out,"%le %le %le %le\n",d[2][2],d[2][3],d[2][4],d[2][5]);
      fprintf(Out,"%le %le %le\n",d[3][3],d[3][4],d[3][5]);
      fprintf(Out,"%le %le\n",d[4][4],d[4][5]);
      fprintf(Out,"%le\n",d[5][5]);
      break;
    }
    default:{
      print_err("unknown type of strain-stress state for mechanical homogeniaztion is required", __FILE__, __LINE__, __func__);
    }
    }//  end of the switch (strastre){
  }
}

/**
   function solves homogenization for mechanical problems on PUC and returns overall (homogenized) stiffness matrix 

   @param unkn_r   - displacements and strains and stresses in integration point 
                     (in master node on PUC) stored as an array (one dimensional)
   @param matrix_k - homogenized stiffness matrix of PUC stored as an array (one dimensional)

   TKr, 07/01/2015
*/
void paral_transport_homogenization (double *unkn_r,double *matrix_k)
{
  //has to be completed??!!
}
